;; Generated package description from realgud.el  -*- no-byte-compile: t; lexical-binding:t -*-
(define-package "realgud" "1.6.0.0.20260303.215436" "A modular front-end for interacting with external debuggers" '((load-relative "1.3.2") (loc-changes "1.2") (test-simple "1.3.0") (emacs "27")) :commit "6a3c23764353a5f94a35f212a482b12346ea03d4" :authors '(("Rocky Bernstein" . "rocky@gnu.org")) :maintainer '("Rocky Bernstein" . "rocky@gnu.org") :keywords '("debugger" "gdb" "python" "perl" "go" "bash" "zsh" "bashdb" "zshdb" "remake" "trepan" "perldb" "pdb") :url "https://github.com/realgud/realgud/")
